#include <rootvis.h>
#include <xmms/configfile.h>

void color_triplet2arr(char* res, char* triplet) {
	if (sscanf(triplet, "#%2hhx%2hhx%2hhx", &res[0], &res[1], &res[2]) != 3)
		fprintf(stderr, "Color value %s could not be recognized as #rrggbb, ranging from #000000 to #FFFFFF\n", triplet);
/*	if (sscanf(triplet, "(%hhd,%hhd,%hhd)", &res[0], &res[1], &res[2]) != 3)
		fprintf(stderr, "Color value %s could not be recognized in the value (x,y,z) whereby x,y,z are between 0 and 255\n", triplet); */
}

char* color_arr2triplet(unsigned char* src, char* triplet) {
	sprintf(triplet, "#%2.2hhx%2.2hhx%2.2hhx", src[0], src[1], src[2]);
	return triplet;
}

void config_write(void) {
	ConfigFile *fp;
	gchar *filename;
	char colortmp[7];
	print_status("Writing configuration");
	filename = g_strconcat(getenv("HOME"), "/.xmms/config", NULL);
	fp = xmms_cfg_open_file(filename);
	if (fp) {
		xmms_cfg_write_int(fp, "rootvis", "stereo", conf.stereo);
		xmms_cfg_write_int(fp, "rootvis", "geometry_posx", conf.geo[0].posx);
		xmms_cfg_write_int(fp, "rootvis", "geometry_posy", conf.geo[0].posy);
		xmms_cfg_write_int(fp, "rootvis", "geometry_orientation", conf.geo[0].orientation);
		xmms_cfg_write_int(fp, "rootvis", "geometry_height", conf.geo[0].height);
		xmms_cfg_write_int(fp, "rootvis", "geometry_space", conf.geo[0].space);
		xmms_cfg_write_int(fp, "rootvis", "bar_width", conf.bar[0].width);
		xmms_cfg_write_int(fp, "rootvis", "bar_shadow", conf.bar[0].shadow);
		xmms_cfg_write_int(fp, "rootvis", "bar_falloff", conf.bar[0].falloff);
		xmms_cfg_write_int(fp, "rootvis", "peak_enabled", conf.peak[0].enabled);
		xmms_cfg_write_int(fp, "rootvis", "peak_falloff", conf.peak[0].falloff);
		xmms_cfg_write_int(fp, "rootvis", "peak_step", conf.peak[0].step);
		xmms_cfg_write_int(fp, "rootvis2", "geometry_posx", conf.geo[1].posx);
		xmms_cfg_write_int(fp, "rootvis2", "geometry_posy", conf.geo[1].posy);
		xmms_cfg_write_int(fp, "rootvis2", "geometry_orientation", conf.geo[1].orientation);
		xmms_cfg_write_int(fp, "rootvis2", "geometry_height", conf.geo[1].height);
		xmms_cfg_write_int(fp, "rootvis2", "geometry_space", conf.geo[1].space);
		xmms_cfg_write_int(fp, "rootvis2", "bar_width", conf.bar[1].width);
		xmms_cfg_write_int(fp, "rootvis2", "bar_shadow", conf.bar[1].shadow);
		xmms_cfg_write_int(fp, "rootvis2", "bar_falloff", conf.bar[1].falloff);
		xmms_cfg_write_int(fp, "rootvis2", "peak_enabled", conf.peak[1].enabled);
		xmms_cfg_write_int(fp, "rootvis2", "peak_falloff", conf.peak[1].falloff);
		xmms_cfg_write_int(fp, "rootvis2", "peak_step", conf.peak[1].step);
		xmms_cfg_write_int(fp, "rootvis", "cutoff", conf.cutoff);
		xmms_cfg_write_int(fp, "rootvis", "div", conf.div);
		xmms_cfg_write_float(fp, "rootvis", "linearity", conf.linearity);
		xmms_cfg_write_int(fp, "rootvis", "fps", conf.fps);
		xmms_cfg_write_string(fp, "rootvis", "bar_color_1", color_arr2triplet(conf.bar[0].color[0], colortmp));
		xmms_cfg_write_string(fp, "rootvis", "bar_color_2", color_arr2triplet(conf.bar[0].color[1], colortmp));
		xmms_cfg_write_string(fp, "rootvis", "bar_color_3", color_arr2triplet(conf.bar[0].color[2], colortmp));
		xmms_cfg_write_string(fp, "rootvis", "bar_color_4", color_arr2triplet(conf.bar[0].color[3], colortmp));
		xmms_cfg_write_string(fp, "rootvis", "bar_shadow_color", color_arr2triplet(conf.bar[0].shadow_color, colortmp));
		xmms_cfg_write_string(fp, "rootvis", "peak_color", color_arr2triplet(conf.peak[0].color, colortmp));
		xmms_cfg_write_string(fp, "rootvis2", "bar_color_1", color_arr2triplet(conf.bar[1].color[0], colortmp));
		xmms_cfg_write_string(fp, "rootvis2", "bar_color_2", color_arr2triplet(conf.bar[1].color[1], colortmp));
		xmms_cfg_write_string(fp, "rootvis2", "bar_color_3", color_arr2triplet(conf.bar[1].color[2], colortmp));
		xmms_cfg_write_string(fp, "rootvis2", "bar_color_4", color_arr2triplet(conf.bar[1].color[3], colortmp));
		xmms_cfg_write_string(fp, "rootvis2", "bar_shadow_color", color_arr2triplet(conf.bar[1].shadow_color, colortmp));
		xmms_cfg_write_string(fp, "rootvis2", "peak_color", color_arr2triplet(conf.peak[1].color, colortmp));
		xmms_cfg_write_int(fp, "rootvis", "debug", conf.debug);
		xmms_cfg_write_file(fp, filename);
		xmms_cfg_free(fp);
	} else	fprintf(stderr, "Could not open xmms configuration file: %s\n", filename);
	g_free(filename);
}

// this parses ~/.xmms/config
// if a setting is not found, it is created to make it possible to edit the default settings
// after the configuration dialogue is finished, this won't be necessary any more
void config_read(void) {
	int dirty;
	ConfigFile *fp;
	gchar *filename, *colortmp;

	conf.stereo = 0;
	for (dirty = 1; dirty >= 0; dirty--) {
		conf.geo[dirty].posx = DEFAULT_geometry_posx;
		conf.geo[dirty].posy = DEFAULT_geometry_posy + dirty * (DEFAULT_geometry_height + 2); // second channel below first
		conf.geo[dirty].orientation = DEFAULT_geometry_orientation;
		conf.geo[dirty].height = DEFAULT_geometry_height;
		conf.geo[dirty].space = DEFAULT_geometry_space;
		conf.bar[dirty].width = DEFAULT_bar_width;
		conf.bar[dirty].shadow = DEFAULT_bar_shadow;
		conf.bar[dirty].falloff = DEFAULT_bar_falloff;
		color_triplet2arr(conf.bar[dirty].color[0], DEFAULT_bar_color_1);
		color_triplet2arr(conf.bar[dirty].color[1], DEFAULT_bar_color_2);
		color_triplet2arr(conf.bar[dirty].color[2], DEFAULT_bar_color_3);
		color_triplet2arr(conf.bar[dirty].color[3], DEFAULT_bar_color_4);
		color_triplet2arr(conf.bar[dirty].shadow_color, DEFAULT_bar_shadow_color);
		conf.peak[dirty].enabled = DEFAULT_peak_enabled;
		conf.peak[dirty].falloff = DEFAULT_peak_falloff;
		conf.peak[dirty].step = DEFAULT_peak_step;
		color_triplet2arr(conf.peak[dirty].color, DEFAULT_peak_color);
	} // dirty is now set to 0
	conf.cutoff = DEFAULT_cutoff;
	conf.div = DEFAULT_div;
	conf.linearity = DEFAULT_linearity;
	conf.fps = DEFAULT_fps;
	conf.debug = DEFAULT_debug;
	
	print_status("Reading configuration");
	
	/* without gtk it would be
	   filename = strcat(getenv("HOME"), "/.xmms/config");
	   after determining the length */
	filename = g_strconcat(getenv("HOME"), "/.xmms/config", NULL);
	fp = xmms_cfg_open_file(filename);

	if (fp) {
		if (!(xmms_cfg_read_int(fp, "rootvis", "stereo", &conf.stereo))) dirty++;
		if (!(xmms_cfg_read_int(fp, "rootvis", "geometry_posx", &conf.geo[0].posx))) dirty++;
		if (!(xmms_cfg_read_int(fp, "rootvis", "geometry_posy", &conf.geo[0].posy))) dirty++;
		if (!(xmms_cfg_read_int(fp, "rootvis", "geometry_orientation", &conf.geo[0].orientation))) dirty++;
		if (!(xmms_cfg_read_int(fp, "rootvis", "geometry_height", &conf.geo[0].height))) dirty++;
		if (!(xmms_cfg_read_int(fp, "rootvis", "geometry_space", &conf.geo[0].space))) dirty++;
		if (!(xmms_cfg_read_int(fp, "rootvis", "bar_width", &conf.bar[0].width))) dirty++;
		if (!(xmms_cfg_read_int(fp, "rootvis", "bar_shadow", &conf.bar[0].shadow))) dirty++;
		if (!(xmms_cfg_read_int(fp, "rootvis", "bar_falloff", &conf.bar[0].falloff))) dirty++;
		if (!(xmms_cfg_read_int(fp, "rootvis", "peak_enabled", &conf.peak[0].enabled))) dirty++;
		if (!(xmms_cfg_read_int(fp, "rootvis", "peak_falloff", &conf.peak[0].falloff))) dirty++;
		if (!(xmms_cfg_read_int(fp, "rootvis", "peak_step", &conf.peak[0].step))) dirty++;
		if (conf.stereo) {
			if (!(xmms_cfg_read_int(fp, "rootvis2", "geometry_posx", &conf.geo[1].posx))) dirty++;
			if (!(xmms_cfg_read_int(fp, "rootvis2", "geometry_posy", &conf.geo[1].posy))) dirty++;
			if (!(xmms_cfg_read_int(fp, "rootvis2", "geometry_orientation", &conf.geo[1].orientation))) dirty++;
			if (!(xmms_cfg_read_int(fp, "rootvis2", "geometry_height", &conf.geo[1].height))) dirty++;
			if (!(xmms_cfg_read_int(fp, "rootvis2", "geometry_space", &conf.geo[1].space))) dirty++;
			if (!(xmms_cfg_read_int(fp, "rootvis2", "bar_width", &conf.bar[1].width))) dirty++;
			if (!(xmms_cfg_read_int(fp, "rootvis2", "bar_shadow", &conf.bar[1].shadow))) dirty++;
			if (!(xmms_cfg_read_int(fp, "rootvis2", "bar_falloff", &conf.bar[1].falloff))) dirty++;
			if (!(xmms_cfg_read_int(fp, "rootvis2", "peak_enabled", &conf.peak[1].enabled))) dirty++;
			if (!(xmms_cfg_read_int(fp, "rootvis2", "peak_falloff", &conf.peak[1].falloff))) dirty++;
			if (!(xmms_cfg_read_int(fp, "rootvis2", "peak_step", &conf.peak[1].step))) dirty++;
		}
		if (!(xmms_cfg_read_int(fp, "rootvis", "cutoff", &conf.cutoff))) dirty++;
		if (!(xmms_cfg_read_int(fp, "rootvis", "div", &conf.div))) dirty++;
		if (!(xmms_cfg_read_float(fp, "rootvis", "linearity", &conf.linearity))) dirty++;
		if (!(xmms_cfg_read_int(fp, "rootvis", "fps", &conf.fps))) dirty++;
		if (!(xmms_cfg_read_int(fp, "rootvis", "debug", &conf.debug))) dirty++;
		if (!(xmms_cfg_read_string(fp, "rootvis", "bar_color_1", &colortmp))) dirty++;
		 else	color_triplet2arr(conf.bar[0].color[0], colortmp);
		if (!(xmms_cfg_read_string(fp, "rootvis", "bar_color_2", &colortmp))) dirty++;
		 else	color_triplet2arr(conf.bar[0].color[1], colortmp);
		if (!(xmms_cfg_read_string(fp, "rootvis", "bar_color_3", &colortmp))) dirty++;
		 else	color_triplet2arr(conf.bar[0].color[2], colortmp);
		if (!(xmms_cfg_read_string(fp, "rootvis", "bar_color_4", &colortmp))) dirty++;
		 else	color_triplet2arr(conf.bar[0].color[3], colortmp);
		if (!(xmms_cfg_read_string(fp, "rootvis", "bar_shadow_color", &colortmp))) dirty++;
		 else	color_triplet2arr(conf.bar[0].shadow_color, colortmp);
		if (!(xmms_cfg_read_string(fp, "rootvis", "peak_color", &colortmp))) dirty++;
		 else	color_triplet2arr(conf.peak[0].color, colortmp);
		if (conf.stereo) {
			if (!(xmms_cfg_read_string(fp, "rootvis2", "bar_color_1", &colortmp))) dirty++;
			else	color_triplet2arr(conf.bar[1].color[0], colortmp);
			if (!(xmms_cfg_read_string(fp, "rootvis2", "bar_color_2", &colortmp))) dirty++;
			else	color_triplet2arr(conf.bar[1].color[1], colortmp);
			if (!(xmms_cfg_read_string(fp, "rootvis2", "bar_color_3", &colortmp))) dirty++;
			else	color_triplet2arr(conf.bar[1].color[2], colortmp);
			if (!(xmms_cfg_read_string(fp, "rootvis2", "bar_color_4", &colortmp))) dirty++;
			else	color_triplet2arr(conf.bar[1].color[3], colortmp);
			if (!(xmms_cfg_read_string(fp, "rootvis2", "bar_shadow_color", &colortmp))) dirty++;
			else	color_triplet2arr(conf.bar[1].shadow_color, colortmp);
			if (!(xmms_cfg_read_string(fp, "rootvis2", "peak_color", &colortmp))) dirty++;
			else	color_triplet2arr(conf.peak[1].color, colortmp);
		}
		xmms_cfg_free(fp);
		if (dirty > 0) config_write();
	} else	fprintf(stderr, "Could not open xmms configuration file: %s\n", filename);
	g_free(filename);
	print_status("Configuration finished");
}
